import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread";

/* eslint-disable react-hooks/exhaustive-deps */
import { useEffect, useState } from 'react';
import OptimizelyX from './optimizely-x';
export default (function (params) {
  var experimentId = params.experimentId,
      forceActivation = params.forceActivation,
      _params$forceActivati = params.forceActivationDelay,
      forceActivationDelay = _params$forceActivati === void 0 ? 1000 : _params$forceActivati,
      forceVariation = params.forceVariation,
      rawVariations = params.variations; // parse variations config

  var variations = rawVariations.map(function (raw) {
    if (typeof raw === 'object') return raw;
    return {
      id: raw // make an object from a naked id

    };
  }); // get default variation

  var defaultVariation = variations.find(function (variation) {
    return variation.isDefault;
  });
  var defaultVariationId = defaultVariation ? defaultVariation.id : null;

  var getNameFromVariation = function getNameFromVariation(variation) {
    // variationName exists in object
    if (variation.name) return variation.name; // otherwise, use an alphabet letter based on variation position

    var variationIndex = variations.indexOf(variation);
    return String.fromCharCode(65 + variationIndex); // A, B, C, D, ...
  };

  var checkVariationId = function checkVariationId(variationId) {
    var checkedVariation = variationId && variations.find(function (variation) {
      return (// name or id are both valid ways to force a variation
        variationId === getNameFromVariation(variation) || variationId === variation.id
      );
    });
    return checkedVariation ? checkedVariation.id : null;
  };

  var getNameFromVariationId = function getNameFromVariationId(variationId) {
    return getNameFromVariation(variations.find(function (variation) {
      return variationId === variation.id;
    }));
  };

  var getFlagsFromVariationId = function getFlagsFromVariationId(variationId) {
    return variations.reduce(function (obj, variation) {
      var _objectSpread2;

      var variationName = getNameFromVariation(variation); // capitalize variation name in order to get a well-formed camel case key
      // i. e. for a name 'foo' → isVariationFoo

      var capitalizedVariationName = variationName.charAt(0).toUpperCase() + variationName.slice(1);
      return _objectSpread({}, obj, (_objectSpread2 = {}, _objectSpread2["isVariation" + capitalizedVariationName] = variationId === variation.id, _objectSpread2));
    }, {});
  };

  var buildExperimentState = function buildExperimentState(_ref) {
    var variationId = _ref.variationId,
        _ref$active = _ref.active,
        active = _ref$active === void 0 ? false : _ref$active;
    var variationName = getNameFromVariationId(variationId);
    return _objectSpread({}, params, {
      isActive: active,
      isDefault: variationId === defaultVariationId,
      isVariation: variationId !== defaultVariationId,
      isWrapped: true,
      variationId: variationId,
      variationName: variationName
    }, getFlagsFromVariationId(variationId));
  };

  var initialVariationId = checkVariationId(params.forceVariation) || defaultVariationId; // set state which will be used to provide context

  var initialState = buildExperimentState({
    variationId: initialVariationId
  });

  var _useState = useState(initialState),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var parseVariationId = function parseVariationId(rawVariationId) {
    var numberVariationId = Number(rawVariationId);
    return !isNaN(numberVariationId) ? numberVariationId : rawVariationId;
  };

  var activationHandler = function activationHandler(rawVariationId) {
    var variationId = parseVariationId(rawVariationId);
    return setState(buildExperimentState(_objectSpread({}, state, {
      variationId: variationId,
      active: true
    })));
  };

  var logWatchOutMessage = function logWatchOutMessage(message) {
    // eslint-disable-next-line no-console
    if (process.env.NODE_ENV !== 'test') console.warn(message);
  };

  useEffect(function () {
    if (process.env.NODE_ENV !== 'production') {
      var getMessageForParam = function getMessageForParam(param) {
        return "[ExperimentCore] Watch out!! Optimizely response will be ignored because the " + param + " param.";
      };

      if (forceActivation) {
        setTimeout(function () {
          return activationHandler(checkVariationId(forceActivation));
        }, forceActivationDelay);
        logWatchOutMessage(getMessageForParam('forceActivation'));
        return;
      }

      if (forceVariation) {
        logWatchOutMessage(getMessageForParam('forceVariation'));
        return;
      }
    }

    OptimizelyX.addActivationListener(experimentId, activationHandler);
    return function () {
      if (process.env.NODE_ENV !== 'production' && (forceActivation || forceVariation)) {
        return;
      }

      OptimizelyX.removeActivationListener(experimentId, activationHandler);
    };
  }, []);
  return {
    experimentData: state
  };
});